@@ -1,4 +1,3 @@
-import os
 import fcntl
 import os.path
 from hashlib import sha1
@@ -8,15 +7,17 @@ import shutil
 import logging
 import weakref
 import urllib2
-import filetracker
 import email
 
 from sio.workers import ft, _original_cwd
+from sio.workers.elf_loader_patch import _patch_elf_loader
+from sio.workers.util import rmtree
 
 SANDBOXES_BASEDIR = os.environ.get('SIO_SANDBOXES_BASEDIR',
         os.path.expanduser(os.path.join('~', '.sio-sandboxes')))
-SANDBOXES_URL = os.environ.get('SIO_SANDBOXES_URL')
-CHECK_INTERVAL = 3600
+SANDBOXES_URL = os.environ.get('SIO_SANDBOXES_URL',
+                    'http://downloads.sio2project.mimuw.edu.pl/sandboxes')
+CHECK_INTERVAL = int(os.environ.get('SIO_SANDBOXES_CHECK_INTERVAL', 3600))
 
 logger = logging.getLogger(__name__)
 
@@ -105,6 +106,8 @@ class Sandbox(object):
 
     _instances = weakref.WeakValueDictionary()
 
+    required_fixups = set(('elf_loader_patch',))
+
     @classmethod
     def _instance(cls, name):
         i = cls._instances.get(name)
@@ -133,6 +136,9 @@ class Sandbox(object):
         if self._in_context == 0:
             self.lock.unlock()
 
+    def __str__(self):
+        return "<Sandbox: %s at %s>" % (self.name, self.path,)
+
     def _mark_checked(self):
         # We're assuming this is safe enough to be done under
         # a shared lock, as the check will eventually be re-done
@@ -141,7 +147,17 @@ class Sandbox(object):
         open(last_check_file, 'wb').write(str(int(time.time())))
 
     def _should_install_sandbox(self):
+        if not os.path.isdir(self.path):
+            return True
+
         try:
+            fixups_file = os.path.join(self.path, '.fixups_applied')
+            if not os.path.exists(fixups_file):
+                return True
+            current_fixups = set(open(fixups_file).read().split())
+            if not current_fixups.issuperset(self.required_fixups):
+                return True
+
             last_check_file = os.path.join(self.path, '.last_check')
             last_check = int(open(last_check_file).read())
             now_int = int(time.time())
@@ -154,11 +170,13 @@ class Sandbox(object):
             if not expected_hash:
                 raise SandboxError("Server did not return hash for "
                         "the sandbox image '%s'" % self.name)
+            expected_hash = str(expected_hash)
 
             hash_file = os.path.join(self.path, '.hash')
             if not os.path.exists(hash_file):
                 return True
             hash = open(hash_file, 'rb').read().strip()
+            logger.debug("Comparing hashes: %s vs %s.", expected_hash, hash)
             if hash != expected_hash:
                 return True
 
@@ -184,6 +202,25 @@ class Sandbox(object):
             last_modified = int(email.utils.mktime_tz(last_modified))
         return last_modified
 
+    def _apply_fixups(self):
+        operative = {}
+        if 'elf_loader_patch' in self.required_fixups:
+            operative['elf_loader_patch'] = _patch_elf_loader(self.path)
+
+        fixups_file = os.path.join(self.path, '.fixups_applied')
+        open(fixups_file, 'w').write('\n'.join(self.required_fixups))
+
+        operatives_file = os.path.join(self.path, '.fixups_operative')
+        open(operatives_file, 'w').write('\n'.join(
+            [fixup for fixup in operative if operative[fixup]]))
+
+    def has_fixup(self, name):
+        if not hasattr(self, 'operative_fixups'):
+            operatives_file = os.path.join(self.path, '.fixups_operative')
+            self.operative_fixups = open(operatives_file).read().split('\n')
+
+        return name in self.operative_fixups
+
     def _get(self):
         name = self.name
         path = self.path
@@ -204,51 +241,59 @@ class Sandbox(object):
             self.lock.lock_shared()
             return
 
-        logger.info("Downloading sandbox '%s' ...", name)
-
-        if os.path.exists(path):
-            shutil.rmtree(path)
-
-        archive_path = path + '.tar.gz'
-
         try:
-            ft_path = _filetracker_path(name)
-            ft_client = ft.instance()
-            vname = ft_client.get_file(ft_path, archive_path)
-            version = ft_client.file_version(vname)
-        except Exception, e:
-            logger.warning("Failed to download sandbox from filetracker",
-                    exc_info=True)
-            if SANDBOXES_URL:
-                url = SANDBOXES_URL + '/' + _urllib_path(name)
-                logger.info("  trying url: %s", url)
-                local_f = open(archive_path, 'wb')
-                try:
-                    http_f = urllib2.urlopen(url)
-                    shutil.copyfileobj(http_f, local_f)
-                    local_f.close()
-                except:
-                    os.unlink(archive_path)
-                    raise
-                version = self._parse_last_modified(http_f)
-            else:
-                raise SandboxError("Could not download sandbox '%s'" % (name,))
+            logger.info("Downloading sandbox '%s' ...", name)
 
-        logger.info(" extracting ...")
+            if os.path.exists(path):
+                rmtree(path)
 
-        tar = tarfile.open(archive_path, 'r')
-        tar.extractall(SANDBOXES_BASEDIR)
-        os.unlink(archive_path)
+            archive_path = path + '.tar.gz'
 
-        if not os.path.isdir(path):
-            raise SandboxError("Downloaded sandbox archive "
-                    "did not contain expected directory '%s'" % name)
+            try:
+                ft_path = _filetracker_path(name)
+                ft_client = ft.instance()
+                vname = ft_client.get_file(ft_path, archive_path)
+                version = ft_client.file_version(vname)
+            except Exception, e:
+                logger.warning("Failed to download sandbox from filetracker",
+                        exc_info=True)
+                if SANDBOXES_URL:
+                    url = SANDBOXES_URL + '/' + _urllib_path(name)
+                    logger.info("  trying url: %s", url)
+                    local_f = open(archive_path, 'wb')
+                    try:
+                        http_f = urllib2.urlopen(url)
+                        shutil.copyfileobj(http_f, local_f)
+                        local_f.close()
+                    except:
+                        os.unlink(archive_path)
+                        raise
+                    version = self._parse_last_modified(http_f)
+                else:
+                    raise SandboxError("Could not download sandbox '%s'"
+                                        % (name,))
 
-        hash_file = os.path.join(path, '.hash')
-        open(hash_file, 'wb').write(str(version))
+            logger.info(" extracting ...")
 
-        self._mark_checked()
-        logger.info(" done.")
+            tar = tarfile.open(archive_path, 'r')
+            tar.extractall(SANDBOXES_BASEDIR)
+            os.unlink(archive_path)
+
+            if not os.path.isdir(path):
+                raise SandboxError("Downloaded sandbox archive "
+                        "did not contain expected directory '%s'" % name)
+
+            self._apply_fixups()
+
+            hash_file = os.path.join(path, '.hash')
+            open(hash_file, 'wb').write(str(version))
+
+            self._mark_checked()
+            logger.info(" done.")
+
+        except:
+            self.lock.unlock()
+            raise
 
         self.lock.lock_shared()
 
